@@ -27,6 +27,7 @@ sub fatal(@);
 
 my $progname = basename($0);
 my $modified_conf_msg;
+my $exit_status = 0;
 
 sub usage {
     print <<"EOF";
@@ -61,6 +62,7 @@ Valid options are:
    --renamed FROM TO      The package formerly called FROM has been
                             renamed TO; only of interest with --show-moved
                             (multiple permitted)
+   --quiet, -q            Be quiet if no differences were found
 
 Default settings modified by devscripts configuration files:
 $modified_conf_msg
@@ -84,6 +86,8 @@ my $compare_control = 1;
 my $show_moved = 0;
 my $wdiff_opt = '';
 
+my $quiet = 0;
+
 # Next, read read configuration files and then command line
 # The next stuff is boilerplate
 
@@ -180,6 +184,7 @@ while (@ARGV) {
     }
     elsif ($ARGV[0] =~ /^(--dirs|-d)$/) { $ignore_dirs = 0; shift; }
     elsif ($ARGV[0] eq '--nodirs') { $ignore_dirs = 1; shift; }
+    elsif ($ARGV[0] =~ /^(--quiet|-q)$/) { $quiet = 1; shift; }
     elsif ($ARGV[0] =~ /^(--show-moved|-s)$/) { $show_moved = 1; shift; }
     elsif ($ARGV[0] eq '--noshow-moved') { $show_moved = 0; shift; }
     elsif ($ARGV[0] eq '--nocontrol') { $compare_control = 0; shift; }
@@ -210,15 +215,16 @@ if (! $type) {
     }
 
     if ($ARGV[0] =~ /\.deb$/) { $type = 'deb'; }
+    elsif ($ARGV[0] =~ /\.udeb$/) { $type = 'deb'; }
     elsif ($ARGV[0] =~ /\.changes$/) { $type = 'changes'; }
     elsif ($ARGV[0] =~ /\.dsc$/) { $type = 'dsc'; }
     elsif (`file $ARGV[0]` =~ /Debian/) { $type = 'deb'; }
     else {
-	fatal "Could not recognise files; the names should end .deb, .changes or .dsc";
+	fatal "Could not recognise files; the names should end .deb, .udeb, .changes or .dsc";
     }
     if ($ARGV[1] !~ /\.$type$/) {
 	unless ($type eq 'deb' and `file $ARGV[0]` =~ /Debian/) {
-	    fatal "The two filenames must have the same suffix, either .deb, .changes or .dsc";
+	    fatal "The two filenames must have the same suffix, either .deb, .udeb, .changes or .dsc";
 	}
     }
 }
@@ -271,7 +277,7 @@ elsif ($type eq 'changes' or $type eq 'debs') {
 		last if $infiles and /^[^ ]/;
 		/^Files:/ and $infiles=1, next;
 		next unless $infiles;
-		/ (\S*.deb)$/ and push @debs, $1;
+		/ (\S*.u?deb)$/ and push @debs, $1;
 	    }
 	    close CHANGES
 		or fatal "Problem reading $changes: $!";
@@ -358,32 +364,34 @@ elsif ($type eq 'dsc') {
     system("(interdiff --version) >/dev/null 2>&1");
     my $use_interdiff = ($?==0) ? 1 : 0;
 
-    if ($origs[1] eq $origs[2] and $use_interdiff) {
+    if ($origs[1] eq $origs[2] and defined $diffs[1] and defined $diffs[2]
+	and $use_interdiff) {
 	# same orig tar ball and interdiff exists
-	if (defined $diffs[1] and defined $diffs[2]) {
-	    my $rv = system('interdiff', '-z', $diffs[1], $diffs[2]);
-	    if ($rv) {
-		fatal "interdiff -z $diffs[1] $diffs[2] failed!";
-	    }
-	}
-	else {
-	    warn "Warning: no diffs and same tarball ($origs[1]),\nnot attempting to diff\n";
+	my $rv = system('interdiff', '-z', $diffs[1], $diffs[2]);
+	if ($rv) {
+	    fatal "interdiff -z $diffs[1] $diffs[2] failed!";
 	}
     } else {
-	if ($origs[1] eq $origs[2]) {
+	# Any other situation
+	if ($origs[1] eq $origs[2] and
+	    defined $diffs[1] and defined $diffs[2]) {
 	    warn "Warning: You do not seem to have interdiff (in the patchutils package)\ninstalled; this program would use it if it were available.\n";
 	}
-	# different orig tarball
+	# possibly different orig tarballs, or no interdiff installed
 	our ($sdir1, $sdir2);
 	mktmpdirs();
 	for my $i (1,2) {
 	    no strict 'refs';
-	    my $cmd = qq(cd ${"dir$i"} && dpkg-source -x $dscs[$i]);
-	    ${"sdir$i"} = `$cmd`;
-	    fatal qq(cd ${"dir$i"} && dpkg-source -x $dscs[$i] failed)
-		if $? != 0;
-	    ${"sdir$i"} =~ /([^\s]*)$/;
-	    ${"sdir$i"} = $1;
+	    my $cmd = qq(cd ${"dir$i"} && dpkg-source -x $dscs[$i] >/dev/null);
+	    system $cmd;
+	    fatal "$cmd failed" if $? != 0;
+	    opendir DIR,${"dir$i"};
+	    while ($_ = readdir(DIR)) {
+		    next if $_ eq '.' || $_ eq '..' || ! -d ${"dir$i"}."/$_";
+		    ${"sdir$i"} = $_;
+		    last;
+	    }
+	    closedir(DIR);
 	}
 	system ("diff", "-Nru", "$dir1/$sdir1", "$dir2/$sdir2");
     }
@@ -466,7 +474,7 @@ if ($show_moved and $type ne 'deb') {
     # This is not a very efficient way of doing things if there are
     # lots of debs involved, but since that is highly unlikely, it
     # shouldn't be much of an issue
-    my $changes = 0;
+    my $changed = 0;
 
     for my $deb1 (sort(keys %debs1), '-') {
 	next unless exists $changes{$deb1};
@@ -482,13 +490,14 @@ if ($show_moved and $type ne 'deb') {
 	    }
 	    print $msg, "\n", '-' x length $msg, "\n";
 	    print join("\n",@{$changes{$deb1}{$deb2}}), "\n\n";
-	    $changes = 1;
+	    $changed = 1;
 	}
     }
 
-    if (! $changes) {
+    if (! $quiet && ! $changed) {
 	print "File lists identical on package level (after any substitutions)\n";
     }
+    $exit_status = 1 if $changed;
 } else {
     my %files;
     grep $files{$_}--, @D1;
@@ -497,7 +506,7 @@ if ($show_moved and $type ne 'deb') {
     my @losses = sort grep $files{$_} < 0, keys %files;
     my @gains = sort grep $files{$_} > 0, keys %files;
 
-    if (@losses == 0 && @gains == 0) {
+    if (! $quiet && @losses == 0 && @gains == 0) {
 	print "File lists identical (after any substitutions)\n";
     }
 
@@ -511,6 +520,7 @@ if ($show_moved and $type ne 'deb') {
 	}
 	print $msg, "\n", '-' x length $msg, "\n";
 	print join("\n",@gains), "\n";
+	$exit_status = 1;
     }
 
     if (@losses) {
@@ -524,17 +534,18 @@ if ($show_moved and $type ne 'deb') {
 	}
 	print $msg, "\n", '-' x length $msg, "\n";
 	print join("\n",@losses), "\n";
+	$exit_status = 1;
     }
 }
 
 # We compare the control files (at least the dependency fields)
 # if we are examining precisely two .debs.
-exit 0 unless defined $singledeb[1] and defined $singledeb[2]
+exit $exit_status unless defined $singledeb[1] and defined $singledeb[2]
     and $compare_control;
 
 unless (system ("command -v wdiff >/dev/null 2>&1") == 0) {
     warn "Can't compare control files; wdiff package not installed\n";
-    exit 0;
+    exit $exit_status;
 }
 
 mktmpdirs();
@@ -551,11 +562,13 @@ for my $i (1,2) {
 
 use strict 'refs';
 
-print "\n";
 my $wdiff = `wdiff -n $wdiff_opt $dir1/control $dir2/control`;
 if ($? >> 8 == 0) {
-    print "No differences were encountered in the control files\n";
+    if (! $quiet) {
+        print "\nNo differences were encountered in the control files\n";
+    }
 } elsif ($? >> 8 == 1) {
+    print "\n";
     if ($wdiff_opt) {
 	# Don't try messing with control codes
 	my $msg = "The following is the wdiff output between the control files:";
@@ -569,6 +582,7 @@ if ($? >> 8 == 0) {
 	print $msg, "\n", '-' x length $msg, "\n";
 	print join("\n",@output), "\n";
     }
+    $exit_status = 1;
 } else {
     warn "wdiff failed (exit status " . ($? >> 8) .
 	(($? & 0x7f) ? " with signal " . ($? & 0x7f) : "") . ")\n";
@@ -576,6 +590,8 @@ if ($? >> 8 == 0) {
 # Clean up
 system ("rm", "-rf", $dir1, $dir2);
 
+exit $exit_status;
+
 ###### Subroutines
 
 # This routine takes the output of dpkg-deb -c and returns
@@ -588,7 +604,7 @@ sub process_debc($$)
     # Format of dpkg-deb -c output:
     # permissions owner/group size date time name ['->' link destination]
     # And remember the slink -> potato stuff
-    $data =~ s/^(\S+\s+){5}(\S+)( (->|link to) \S+)?\s*$/$2/mg;
+    $data =~ s/^(\S+\s+){5}//mg;
     $data =~ s,^\./,/,mg;
     $data =~ s,^([^/]),/$1,mg;
     @filelist = grep ! m|^/$|, split /\n/, $data; # don't bother keeping '/'
