@@ -67,16 +67,20 @@ Usage: $progname [options] [dir ...]
   Process watchfiles in all .../debian/ subdirs of those listed (or the
   current directory if none listed) to check for upstream releases.
 Options:
-    --report, --no-download
-                   Only report on newer or absent versions, do not download
+    --report       Only report on newer or absent versions, do not download
+    --report-status
+                   Report status of packages, but do not download
     --debug        Dump the downloaded web pages to stdout for debugging
                    your watch file.
     --download     Report on newer and absent versions, and download (default)
     --no-download  Report on newer and absent versions, but don\'t download
     --pasv         Use PASV mode for FTP connections
     --no-pasv      Do not use PASV mode for FTP connections (default)
+    --timeout N    Specifies how much time, in seconds, we give remote
+                   servers to respond (default 20 seconds)
     --symlink      Make an orig.tar.gz symlink to downloaded file (default)
     --rename       Rename to orig.tar.gz instead of symlinking
+                   (Both will use orig.tar.bz2 if appropriate)
     --no-symlink   Don\'t make symlink or rename
     --verbose      Give verbose output
     --no-verbose   Don\'t give verbose output (default)
@@ -133,12 +137,17 @@ our $passive = 'default';
 # The next stuff is boilerplate
 
 my $download = 1;
+my $report = 0; # report even on up-to-date packages?
 my $symlink = 'symlink';
 my $verbose = 0;
 my $check_dirname_level = 1;
 my $check_dirname_regex = 'PACKAGE(-.*)?';
 my $dehs = 0;
 my %dehs_tags;
+my $dehs_end_output = 0;
+my $dehs_start_output = 0;
+my $pkg_report_header = '';
+my $timeout = 20;
 
 if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     $modified_conf_msg = "  (no configuration files read)";
@@ -146,6 +155,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 } else {
     my @config_files = ('/etc/devscripts.conf', '~/.devscripts');
     my %config_vars = (
+		       'USCAN_TIMEOUT' => 20,
 		       'USCAN_DOWNLOAD' => 'yes',
 		       'USCAN_PASV' => 'default',
 		       'USCAN_SYMLINK' => 'symlink',
@@ -173,6 +183,8 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 	or $config_vars{'USCAN_DOWNLOAD'}='yes';
     $config_vars{'USCAN_PASV'} =~ /^(yes|no|default)$/
 	or $config_vars{'USCAN_PASV'}='default';
+    $config_vars{'USCAN_TIMEOUT'} =~ m/^\d+$/
+	or $config_vars{'USCAN_TIMEOUT'}=20;
     $config_vars{'USCAN_SYMLINK'} =~ /^(yes|no|symlinks?|rename)$/
 	or $config_vars{'USCAN_SYMLINK'}='yes';
     $config_vars{'USCAN_SYMLINK'}='symlink'
@@ -196,6 +208,7 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
     $download = $config_vars{'USCAN_DOWNLOAD'} eq 'no' ? 0 : 1;
     $passive = $config_vars{'USCAN_PASV'} eq 'yes' ? 1 :
 	$config_vars{'USCAN_PASV'} eq 'no' ? 0 : 'default';
+    $timeout = $config_vars{'USCAN_TIMEOUT'};
     $symlink = $config_vars{'USCAN_SYMLINK'};
     $verbose = $config_vars{'USCAN_VERBOSE'} eq 'yes' ? 1 : 0;
     $dehs = $config_vars{'USCAN_DEHS_OUTPUT'} eq 'yes' ? 1 : 0;
@@ -205,15 +218,17 @@ if (@ARGV and $ARGV[0] =~ /^--no-?conf$/) {
 
 # Now read the command line arguments
 my $debug = 0;
-my ($opt_h, $opt_v, $opt_download, $opt_passive, $opt_symlink);
+my ($opt_h, $opt_v, $opt_download, $opt_report, $opt_passive, $opt_symlink);
 my ($opt_verbose, $opt_ignore, $opt_level, $opt_regex, $opt_noconf);
-my ($opt_package, $opt_uversion, $opt_watchfile, $opt_dehs);
+my ($opt_package, $opt_uversion, $opt_watchfile, $opt_dehs, $opt_timeout);
 
 GetOptions("help" => \$opt_h,
 	   "version" => \$opt_v,
 	   "download!" => \$opt_download,
 	   "report" => sub { $opt_download = 0; },
+	   "report-status" => sub { $opt_download = 0; $opt_report = 1; },
 	   "passive|pasv!" => \$opt_passive,
+	   "timeout=i" => \$opt_timeout,
 	   "symlink!" => sub { $opt_symlink = $_[1] ? 'symlink' : 'no'; },
 	   "rename" => sub { $opt_symlink = 'rename'; },
 	   "package=s" => \$opt_package,
@@ -239,7 +254,10 @@ if ($opt_v) { version(); exit 0; }
 # Now we can set the other variables according to the command line options
 
 $download = $opt_download if defined $opt_download;
+$report = $opt_report if defined $opt_report;
 $passive = $opt_passive if defined $opt_passive;
+$timeout = $opt_timeout if defined $opt_timeout;
+$timeout = 20 unless defined $timeout and $timeout > 0;
 $symlink = $opt_symlink if defined $opt_symlink;
 $verbose = $opt_verbose if defined $opt_verbose;
 $dehs = $opt_dehs if defined $opt_dehs;
@@ -265,12 +283,18 @@ $check_dirname_regex = $opt_regex if defined $opt_regex;
 if (defined $opt_package) {
     die "$progname: --package requires the use of --upstream-version and --watchfile\nas well; run $progname --help for more details\n"
 	unless defined $opt_uversion and defined $opt_watchfile;
-    $download = 0;
+    $download = -$download unless defined $opt_download;
 }
 
 die "$progname: Can't use --verbose if you're using --dehs!\n"
     if $verbose and $dehs;
 
+die "$progname: Can't use --report-status if you're using --dehs!\n"
+    if $verbose and $report;
+
+die "$progname: Can't use --report-status if you're using --download!\n"
+    if $download and $report;
+
 warn "$progname: You're going to get strange (non-XML) output using --debug and --dehs together!\n"
     if $debug and $dehs;
 
@@ -291,7 +315,7 @@ else { $passive = undef; }
 # to restore $ENV{'FTP_PASSIVE'} to what it was at this point
 
 my $user_agent = LWP::UserAgent->new(env_proxy => 1);
-$user_agent->timeout("20");
+$user_agent->timeout($timeout);
 
 if (defined $opt_watchfile) {
     die "Can't have directory arguments if using --watchfile" if @ARGV;
@@ -352,6 +376,7 @@ if (defined $opt_watchfile) {
     }
 
     # Are there any warnings to give if we're using dehs?
+    $dehs_end_output=1;
     dehs_output if $dehs;
     exit 0;
 }
@@ -581,8 +606,8 @@ sub process_watchline ($$$$$$)
 	    $site = "ftp://$site";
 	    if ($pattern !~ /\(.*\)/) {
 		# watch_version=1 and old style watchfile;
-		# pattern uses ? and * shell wildcards; everything from the first
-		# to last of these metachars is the pattern to match on.
+		# pattern uses ? and * shell wildcards; everything from the
+		# first to last of these metachars is the pattern to match on
 		$pattern =~ s/(\?|\*)/($1/;
 		$pattern =~ s/(\?|\*)([^\?\*]*)$/$1)$2/;
 		$pattern =~ s/\./\\./g;
@@ -642,6 +667,8 @@ sub process_watchline ($$$$$$)
 	    $base =~ s%/[^/]+$%/%;
 	}
 
+	# Handle sf.net addresses specially
+	$base =~ s%^http://sf.net/%http://qa.debian.org/watch/sf.php/%;
 	if ($base =~ m%^(\w+://[^/]+)%) {
 	    $site = $1;
 	} else {
@@ -703,11 +730,22 @@ sub process_watchline ($$$$$$)
 	while ($content =~ m/<\s*a\s+[^>]*href\s*=\s*([\"\'])(.*?)\1/gi) {
 	    my $href = $2;
 	    if ($href =~ m&^$pattern$&) {
-		my $mangled_version = join(".", $href =~ m&^$pattern$&);
-		foreach my $pat (@{$options{'uversionmangle'}}) {
-		    eval "\$mangled_version =~ $pat;";
+		if ($watch_version == 2) {
+		    # watch_version 2 only recognised one group; the code
+		    # below will break version 2 watchfiles with a construction
+		    # such as file-([\d\.]+(-\d+)?) (bug #327258)
+		    push @hrefs, [$1, $href];
+		} else {
+		    # need the map { ... } here to handle cases of (...)?
+		    # which may match but then return undef values
+		    my $mangled_version =
+			join(".", map { $_ if defined($_) }
+			     $href =~ m&^$pattern$&);
+		    foreach my $pat (@{$options{'uversionmangle'}}) {
+			eval "\$mangled_version =~ $pat;";
+		    }
+		    push @hrefs, [$mangled_version, $href];
 		}
-		push @hrefs, [$mangled_version, $href];
 	    }
 	}
 	if (@hrefs) {
@@ -841,18 +879,28 @@ EOF
     $dehs_tags{'upstream-version'} = $newversion;
     $dehs_tags{'upstream-url'} = $upstream_url;
 
-    print "Newest version on remote site is $newversion, local version is $lastversion\n" .
-	($mangled_lastversion eq $lastversion ? "" : " (mangled local version number $mangled_lastversion)\n")
-	if $verbose;
-
     # Can't just use $lastversion eq $newversion, as then 0.01 and 0.1
     # compare different, whereas they are treated as equal by dpkg
     if (system("dpkg --compare-versions '$mangled_lastversion' eq '$newversion'") == 0) {
-	print " => Package is up to date\n" if $verbose;
+	if ($verbose or ($download == 0 and $report and ! $dehs)) {
+	    print $pkg_report_header;
+	    $pkg_report_header = '';
+	    print "Newest version on remote site is $newversion, local version is $lastversion\n" .
+		($mangled_lastversion eq $lastversion ? "" : " (mangled local version number $mangled_lastversion)\n");
+	    print " => Package is up to date\n";
+	}
 	$dehs_tags{'status'} = "up to date";
 	return 0;
     }
 
+    # In all other cases, we'll want to report information even with --report
+    if ($verbose or ($download == 0 and ! $dehs)) {
+	print $pkg_report_header;
+	$pkg_report_header = '';
+	print "Newest version on remote site is $newversion, local version is $lastversion\n" .
+	    ($mangled_lastversion eq $lastversion ? "" : " (mangled local version number $mangled_lastversion)\n");
+    }
+
     # We use dpkg's rules to determine whether our current version
     # is newer or older than the remote version.
     if (system("dpkg --compare-versions '$mangled_lastversion' gt '$newversion'") == 0) {
@@ -873,12 +921,17 @@ EOF
     if (defined $pkg_dir) {
 	if (-f "../$newfile_base") {
 	    print " => $newfile_base already in package directory\n"
-		if $verbose;
+		if $verbose or ($download == 0 and ! $dehs);
 	    return 0;
 	}
 	if (-f "../${pkg}_${newversion}.orig.tar.gz") {
 	    print " => ${pkg}_${newversion}.orig.tar.gz already in package directory\n"
-		if $verbose;
+		if $verbose or ($download == 0 and ! $dehs);
+	    return 0;
+	}
+	elsif (-f "../${pkg}_${newversion}.orig.tar.bz2") {
+	    print " => ${pkg}_${newversion}.orig.tar.bz2 already in package directory\n"
+		if $verbose or ($download == 0 and ! $dehs);
 	    return 0;
 	}
     }
@@ -894,6 +947,15 @@ EOF
 	    ")\n";
     }
 
+    if ($download < 0) {
+	my $msg = "Not downloading as --package was used.  Use --download to force downloading.";
+	if ($dehs) {
+	    dehs_msg($msg);
+	} else {
+	    print "$msg\n";
+	}
+	return 0;
+    }
     return 0 unless $download;
 
     print "-- Downloading updated package $newfile_base\n" if $verbose;
@@ -932,6 +994,13 @@ EOF
 	    move "../$newfile_base", "../${pkg}_${newversion}.orig.tar.gz";
 	}
     }
+    elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	if ($symlink eq 'symlink') {
+	    symlink $newfile_base, "../${pkg}_${newversion}.orig.tar.bz2";
+	} elsif ($symlink eq 'rename') {
+	    move "../$newfile_base", "../${pkg}_${newversion}.orig.tar.bz2";
+	}
+    }
 
     if ($verbose) {
 	print "-- Successfully downloaded updated package $newfile_base\n";
@@ -942,6 +1011,13 @@ EOF
 		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
 	    }
 	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    if ($symlink eq 'symlink') {
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
+	    } elsif ($symlink eq 'rename') {
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
+	    }
+	}
     } elsif ($dehs) {
 	my $msg = "Successfully downloaded updated package $newfile_base";
 	if ($newfile_base =~ /\.(tar\.gz|tgz)$/) {
@@ -951,6 +1027,13 @@ EOF
 		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.gz";
 	    }
 	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    if ($symlink eq 'symlink') {
+		$msg .= " and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it";
+	    } elsif ($symlink eq 'rename') {
+		$msg .= " and renamed it as ${pkg}_${newversion}.orig.tar.bz2";
+	    }
+	}
 	dehs_msg($msg);
     } else {
 	print "$pkg: Successfully downloaded updated package $newfile_base\n";
@@ -961,13 +1044,26 @@ EOF
 		print "    and renamed it as ${pkg}_${newversion}.orig.tar.gz\n";
 	    }
 	}
+	elsif ($newfile_base =~ /\.(tar\.bz2|tbz2?)$/) {
+	    if ($symlink eq 'symlink') {
+		print "    and symlinked ${pkg}_${newversion}.orig.tar.bz2 to it\n";
+	    } elsif ($symlink eq 'rename') {
+		print "    and renamed it as ${pkg}_${newversion}.orig.tar.bz2\n";
+	    }
+	}
     }
 
     # Do whatever the user wishes to do
     if ($action) {
-	my $usefile = ($symlink =~ /^(symlink|rename)$/
-		       and $newfile_base =~ /\.(tar\.gz|tgz)$/) ?
-	    "../${pkg}_${newversion}.orig.tar.gz" : "../$newfile_base";
+	my $usefile = "../$newfile_base";
+	if ($symlink =~ /^(symlink|rename)$/
+	    and $newfile_base =~ /\.(tar\.gz|tgz)$/) {
+	    $usefile = "../${pkg}_${newversion}.orig.tar.gz";
+	}
+	elsif ($symlink =~ /^(symlink|rename)$/
+	    and $newfile_base =~ /\.(tar\.bz2|tbz2)$/) {
+	    $usefile = "../${pkg}_${newversion}.orig.tar.bz2";
+	}
 
 	if ($watch_version > 1) {
 	    print "-- Executing user specified script\n     $action --upstream-version $newversion $newfile_base" if $verbose;
@@ -1179,7 +1275,12 @@ sub process_watchfile ($$$$)
 
 	# Handle shell \\ -> \
 	s/\\\\/\\/g if $watch_version==1;
-	print "-- In $watchfile, processing watchfile line:\n   $_\n" if $verbose;
+	if ($verbose) {
+	    print "-- In $watchfile, processing watchfile line:\n   $_\n";
+	} elsif ($download == 0 and ! $dehs) {
+	    $pkg_report_header = "Processing watchfile line for package $package...\n";
+	}
+	    
 	$status +=
 	    process_watchline($_, $watch_version, $dir, $package, $version,
 			      $watchfile);
@@ -1213,6 +1314,7 @@ sub dehs_die ($)
     my $msg = $_[0];
     $msg =~ s/\s*$//;
     %dehs_tags = ('errors' => "$msg");
+    $dehs_end_output=1;
     dehs_output;
     exit 1;
 }
@@ -1220,26 +1322,32 @@ sub dehs_die ($)
 sub dehs_output ()
 {
     return unless $dehs;
-    my $output = 0;
+
+    if (! $dehs_start_output) {
+	print "<dehs>\n";
+	$dehs_start_output=1;
+    }
 
     for my $tag (qw(package debian-uversion debian-mangled-uversion
 		    upstream-version upstream-url
 		    status messages warnings errors)) {
 	if (exists $dehs_tags{$tag}) {
-	    if (! $output) {
-		print "<dehs>\n";
-		$output = 1;
-	    }
 	    if (ref $dehs_tags{$tag} eq "ARRAY") {
 		foreach my $entry (@{$dehs_tags{$tag}}) {
-		    print "<$tag>$entry</$tag>\n";
+            $entry =~ s/</&lt;/g;
+            $entry =~ s/>/&gt;/g;
+		    $entry =~ s/&/&amp;/g;
+            print "<$tag>$entry</$tag>\n";
 		}
 	    } else {
+            $dehs_tags{$tag} =~ s/</&lt;/g;
+            $dehs_tags{$tag} =~ s/>/&gt;/g;
+		    $dehs_tags{$tag} =~ s/&/&amp;/g;
 		print "<$tag>$dehs_tags{$tag}</$tag>\n";
 	    }
 	}
     }
-    if ($output) {
+    if ($dehs_end_output) {
 	print "</dehs>\n";
     }
 
